@@ -169,7 +169,10 @@ getSection <- function(sectionHeader, outfiletext, headers="standard") {
     "TESTS OF MODEL FIT", "MODEL FIT INFORMATION", "CLASSIFICATION QUALITY",
     "FINAL CLASS COUNTS AND PROPORTIONS FOR THE LATENT CLASSES",
     "FINAL CLASS COUNTS AND PROPORTIONS FOR THE LATENT CLASS PATTERNS",
+    "LATENT TRANSITION PROBABILITIES BASED ON THE ESTIMATED MODEL",
+    "FINAL CLASS COUNTS AND PROPORTIONS FOR EACH LATENT CLASS VARIABLE",
     "CLASSIFICATION OF INDIVIDUALS BASED ON THEIR MOST LIKELY LATENT CLASS MEMBERSHIP",
+    "Average Latent Class Probabilities for Most Likely Latent Class Membership \\(Row\\)",
     "MODEL RESULTS", "LOGISTIC REGRESSION ODDS RATIO RESULTS", "RESULTS IN PROBABILITY SCALE",
     "IRT PARAMETERIZATION IN TWO-PARAMETER LOGISTIC METRIC",
     "LATENT CLASS ODDS RATIO RESULTS", "LOGRANK OUTPUT", "STANDARDIZED MODEL RESULTS", 
@@ -186,7 +189,8 @@ getSection <- function(sectionHeader, outfiletext, headers="standard") {
     "EQUALITY TESTS OF MEANS ACROSS CLASSES USING POSTERIOR PROBABILITY-BASED",
     "THE FOLLOWING DATA SET\\(S\\) DID NOT RESULT IN A COMPLETED REPLICATION:",
     "RESIDUAL OUTPUT", "MODEL MODIFICATION INDICES", "MODEL COMMAND WITH FINAL ESTIMATES USED AS STARTING VALUES",
-    "FACTOR SCORE INFORMATION (COMPLETE DATA)", "SUMMARY OF FACTOR SCORES", "PLOT INFORMATION", "SAVEDATA INFORMATION",
+    "FACTOR SCORE INFORMATION \\(COMPLETE DATA\\)", "SUMMARY OF FACTOR SCORES", "PLOT INFORMATION", "SAVEDATA INFORMATION",
+    "SAMPLE STATISTICS FOR ESTIMATED FACTOR SCORES",
     "Beginning Time:\\s*\\d+:\\d+:\\d+", "MUTHEN & MUTHEN")
 
 
